#pragma once

/* build batch file */
/*
#ifdef APE_BATCH_FILE_VERSION
Set _MAV=1262
#endif
*/

/* major version number */
#define APE_VERSION_MAJOR 12

/* build version number */
#define APE_VERSION_REVISION 62

/* library interface version, update this whenever the signature of an exported function changes */
#define APE_INTERFACE_VERSION 15

/* leave this so the end of file doesn't get truncated */
